Class: GitHubChangelogGenerator::ChangelogGenerator
- Inherits:
-
Object
- Object
- GitHubChangelogGenerator::ChangelogGenerator
- Defined in:
- lib/github_changelog_generator.rb
Constant Summary collapse
- PER_PAGE_NUMBER =
30
Instance Attribute Summary collapse
-
#all_tags ⇒ Object
Returns the value of attribute all_tags.
-
#github ⇒ Object
Returns the value of attribute github.
-
#options ⇒ Object
Returns the value of attribute options.
Instance Method Summary collapse
- #compund_changelog ⇒ Object
- #create_log(pull_requests, issues, newer_tag, older_tag_name = nil) ⇒ String
- #delete_by_time(array, hash_key, older_tag = nil, newer_tag = nil) ⇒ Object
- #detect_actual_closed_dates ⇒ Object
- #exec_command(cmd) ⇒ Object
- #fetch_event_for_issues(filtered_issues) ⇒ Object
- #fetch_issues_and_pull_requests ⇒ Object
- #fetch_merged_at_pull_requests ⇒ Object
- #fetch_tags_dates ⇒ Object
- #filter_by_milestone(filtered_issues, newer_tag_name, src_array) ⇒ Object
- #find_closed_date_by_commit(issue) ⇒ Object
- #generate_header(log, newer_tag_name, newer_tag_name2, newer_tag_time, older_tag_name, project_url) ⇒ Object
- #generate_log_between_tags(older_tag, newer_tag) ⇒ Object
- #generate_log_for_all_tags ⇒ Object
- #generate_log_from_array(issues, prefix) ⇒ Object
- #get_all_tags ⇒ Object
- #get_filtered_issues ⇒ Object
- #get_filtered_pull_requests ⇒ Object
- #get_time_of_tag(tag_name, tag_times_hash = @tag_times_hash) ⇒ Object
- #github_token ⇒ Object
-
#initialize ⇒ ChangelogGenerator
constructor
A new instance of ChangelogGenerator.
- #is_megred(number) ⇒ Object
- #print_json(json) ⇒ Object
Constructor Details
#initialize ⇒ ChangelogGenerator
Returns a new instance of ChangelogGenerator.
19 20 21 22 23 24 25 26 27 28 29 30 31 32 33 34 35 36 37 38 39 40 41 42 43 44 45 46 47 48 49 50 51 52 53 54 55 56 57 58 59 |
# File 'lib/github_changelog_generator.rb', line 19 def initialize = Parser. if [:verbose] puts 'Input options:' pp puts '' end github_token = {per_page: PER_PAGE_NUMBER} [:oauth_token] = @github_token unless @github_token.nil? [:endpoint] = [:github_endpoint] unless [:github_endpoint].nil? [:site] = [:github_endpoint] unless [:github_site].nil? @github = Github.new @generator = Generator.new() = self. @issues, @pull_requests = self.fetch_issues_and_pull_requests if [:pulls] @pull_requests = self.get_filtered_pull_requests self.fetch_merged_at_pull_requests else @pull_requests = [] end if [:issues] @issues = self.get_filtered_issues fetch_event_for_issues(@issues) detect_actual_closed_dates else @issues = [] end @tag_times_hash = {} end |
Instance Attribute Details
#all_tags ⇒ Object
Returns the value of attribute all_tags.
15 16 17 |
# File 'lib/github_changelog_generator.rb', line 15 def end |
#github ⇒ Object
Returns the value of attribute github.
15 16 17 |
# File 'lib/github_changelog_generator.rb', line 15 def github @github end |
#options ⇒ Object
Returns the value of attribute options.
15 16 17 |
# File 'lib/github_changelog_generator.rb', line 15 def end |
Instance Method Details
#compund_changelog ⇒ Object
208 209 210 211 212 213 214 215 216 217 218 219 220 221 222 223 224 225 226 227 228 229 230 231 232 233 234 235 236 237 238 239 240 241 242 243 244 245 246 |
# File 'lib/github_changelog_generator.rb', line 208 def compund_changelog log = "# Change Log\n\n" if [:unreleased_only] log += self.(self.[0], nil) elsif [:tag1] and [:tag2] tag1 = [:tag1] tag2 = [:tag2] = [] self..each { |x| .push(x['name']) } if .include?(tag1) if .include?(tag2) to_a = .map.with_index.to_a hash = Hash[to_a] index1 = hash[tag1] index2 = hash[tag2] log += self.(self.[index1], self.[index2]) else puts "Can't find tag #{tag2} -> exit" exit end else puts "Can't find tag #{tag1} -> exit" exit end else log += self. end log += "\n\n\\* *This Change Log was automatically generated by [github_changelog_generator](https://github.com/skywinder/Github-Changelog-Generator)*" output_filename = "#{@options[:output]}" File.open(output_filename, 'w') { |file| file.write(log) } puts 'Done!' puts "Generated log placed in #{`pwd`.strip!}/#{output_filename}" end |
#create_log(pull_requests, issues, newer_tag, older_tag_name = nil) ⇒ String
449 450 451 452 453 454 455 456 457 458 459 460 461 462 463 464 465 466 467 468 469 470 471 472 473 474 475 476 477 478 479 480 481 482 483 484 485 486 487 488 489 490 491 492 493 494 495 496 497 498 499 500 501 502 503 504 505 506 |
# File 'lib/github_changelog_generator.rb', line 449 def create_log(pull_requests, issues, newer_tag, older_tag_name = nil) newer_tag_time = newer_tag.nil? ? nil : self.get_time_of_tag(newer_tag) newer_tag_name = newer_tag.nil? ? nil : newer_tag['name'] github_site = [:github_site] || 'https://github.com' project_url = "#{github_site}/#{@options[:user]}/#{@options[:project]}" if newer_tag.nil? newer_tag_name = 'Unreleased' newer_tag_link = 'HEAD' newer_tag_time = Time.new else newer_tag_link = newer_tag_name end log = '' log += generate_header(log, newer_tag_name, newer_tag_link, newer_tag_time, older_tag_name, project_url) if [:issues] # Generate issues: issues_a = [] enhancement_a = [] bugs_a =[] issues.each { |dict| added = false dict.labels.each { |label| if label.name == 'bug' bugs_a.push dict added = true next end if label.name == 'enhancement' enhancement_a.push dict added = true next end } unless added issues_a.push dict end } log += generate_log_from_array(enhancement_a, [:enhancement_prefix]) log += generate_log_from_array(bugs_a, [:bug_prefix]) log += generate_log_from_array(issues_a, [:issue_prefix]) if [:pulls] # Generate pull requests: log += generate_log_from_array(pull_requests, [:merge_prefix]) end end log end |
#delete_by_time(array, hash_key, older_tag = nil, newer_tag = nil) ⇒ Object
412 413 414 415 416 417 418 419 420 421 422 423 424 425 426 427 428 429 430 431 432 433 434 435 436 437 438 439 440 441 442 443 |
# File 'lib/github_changelog_generator.rb', line 412 def delete_by_time(array, hash_key, older_tag = nil, newer_tag = nil) raise 'At least on of the tags should be not nil!' if (older_tag.nil? && newer_tag.nil?) newer_tag_time = self.get_time_of_tag(newer_tag) older_tag_time = self.get_time_of_tag(older_tag) array.select { |req| if req[hash_key] t = Time.parse(req[hash_key]).utc if older_tag_time.nil? tag_in_range_old = true else tag_in_range_old = t > older_tag_time end if newer_tag_time.nil? tag_in_range_new = true else tag_in_range_new = t <= newer_tag_time end tag_in_range = (tag_in_range_old) && (tag_in_range_new) tag_in_range else false end } end |
#detect_actual_closed_dates ⇒ Object
61 62 63 64 65 66 67 68 69 70 71 72 73 74 75 76 77 78 |
# File 'lib/github_changelog_generator.rb', line 61 def detect_actual_closed_dates if [:verbose] print "Fetching closed dates for issues...\r" end threads = [] @issues.each { |issue| threads << Thread.new { find_closed_date_by_commit(issue) } } threads.each { |thr| thr.join } if [:verbose] puts 'Fetching closed dates for issues: Done!' end end |
#exec_command(cmd) ⇒ Object
102 103 104 105 |
# File 'lib/github_changelog_generator.rb', line 102 def exec_command(cmd) exec_cmd = "cd #{$project_path} and #{cmd}" %x[#{exec_cmd}] end |
#fetch_event_for_issues(filtered_issues) ⇒ Object
623 624 625 626 627 628 629 630 631 632 633 634 635 636 637 638 639 640 641 642 643 644 645 |
# File 'lib/github_changelog_generator.rb', line 623 def fetch_event_for_issues(filtered_issues) if [:verbose] print "Fetching events for issues: 0/#{filtered_issues.count}\r" end # Async fetching events: threads = [] i = 0 filtered_issues.each { |issue| threads << Thread.new { obj = @github.issues.events.list user: [:user], repo: [:project], issue_number: issue['number'] issue[:events] = obj.body print "Fetching events for issues: #{i+1}/#{filtered_issues.count}\r" i +=1 } } threads.each { |thr| thr.join } if [:verbose] puts "Fetching events for issues: Done!" end end |
#fetch_issues_and_pull_requests ⇒ Object
591 592 593 594 595 596 597 598 599 600 601 602 603 604 605 606 607 608 609 610 611 612 613 614 615 616 617 618 619 620 621 |
# File 'lib/github_changelog_generator.rb', line 591 def fetch_issues_and_pull_requests if [:verbose] print "Fetching closed issues...\r" end response = @github.issues.list user: [:user], repo: [:project], state: 'closed', filter: 'all', labels: nil issues = [] page_i = 0 count_pages = response.count_pages response.each_page do |page| page_i += PER_PAGE_NUMBER print "Fetching issues... #{page_i}/#{count_pages * PER_PAGE_NUMBER}\r" issues.concat(page) end print " \r" if [:verbose] puts "Received issues: #{issues.count}" end # remove pull request from issues: issues_wo_pr = issues.select { |x| x.pull_request == nil } pull_requests = issues.select { |x| x.pull_request != nil } return issues_wo_pr, pull_requests end |
#fetch_merged_at_pull_requests ⇒ Object
107 108 109 110 111 112 113 114 115 116 117 118 119 120 121 122 123 124 125 126 127 128 129 130 131 132 133 |
# File 'lib/github_changelog_generator.rb', line 107 def fetch_merged_at_pull_requests if [:verbose] print "Fetching pull requests...\r" end response = @github.pull_requests.list [:user], [:project], :state => 'closed' pull_requests = [] page_i = 0 response.each_page do |page| page_i += PER_PAGE_NUMBER count_pages = response.count_pages print "Fetching pull requests... #{page_i}/#{count_pages * PER_PAGE_NUMBER}\r" pull_requests.concat(page) end print " \r" if [:verbose] puts "Received pull requests: #{pull_requests.count}" end @pull_requests.each { |pr| fetched_pr = pull_requests.find { |fpr| fpr.number == pr.number } pr[:merged_at] = fetched_pr[:merged_at] pull_requests.delete(fetched_pr) } end |
#fetch_tags_dates ⇒ Object
281 282 283 284 285 286 287 288 289 290 291 292 293 294 295 296 297 298 299 300 301 302 303 304 305 306 307 308 309 |
# File 'lib/github_changelog_generator.rb', line 281 def if [:verbose] print "Fetching tags dates..\r" end # Async fetching tags: threads = [] i = 0 all = .count .each { |tag| # explicit set @tag_times_hash to write data safety. threads << Thread.new { self.get_time_of_tag(tag, @tag_times_hash) if [:verbose] print "Fetching tags dates: #{i+1}/#{all}\r" i+=1 end } } print " \r" threads.each { |thr| thr.join } if [:verbose] puts 'Fetching tags: Done!' end end |
#filter_by_milestone(filtered_issues, newer_tag_name, src_array) ⇒ Object
377 378 379 380 381 382 383 384 385 386 387 388 389 390 391 392 393 394 395 396 397 398 399 400 401 402 403 404 405 406 407 408 409 410 |
# File 'lib/github_changelog_generator.rb', line 377 def filter_by_milestone(filtered_issues, newer_tag_name, src_array) filtered_issues.select! { |issue| # leave issues without milestones if issue.milestone.nil? true else #check, that this milestone in tag list: .find { |tag| tag.name == issue.milestone.title }.nil? end } unless newer_tag_name.nil? #add missed issues (according milestones) issues_to_add = src_array.select { |issue| if issue.milestone.nil? false else #check, that this milestone in tag list: milestone_is_tag = .find { |tag| tag.name == issue.milestone.title } if milestone_is_tag.nil? false else issue.milestone.title == newer_tag_name end end } filtered_issues |= issues_to_add end filtered_issues end |
#find_closed_date_by_commit(issue) ⇒ Object
80 81 82 83 84 85 86 87 88 89 90 91 92 93 94 95 96 |
# File 'lib/github_changelog_generator.rb', line 80 def find_closed_date_by_commit(issue) unless issue['events'].nil? # reverse! - to find latest closed event. (event goes in date order) issue['events'].reverse!.each { |event| if event[:event].eql? 'closed' if event[:commit_id].nil? issue[:actual_date] = issue[:closed_at] else commit = @github.git_data.commits.get [:user], [:project], event[:commit_id] issue[:actual_date] = commit[:author][:date] end break end } end #TODO: assert issues, that remain without 'actual_date' hash for some reason. end |
#generate_header(log, newer_tag_name, newer_tag_name2, newer_tag_time, older_tag_name, project_url) ⇒ Object
523 524 525 526 527 528 529 530 531 532 533 534 535 536 537 |
# File 'lib/github_changelog_generator.rb', line 523 def generate_header(log, newer_tag_name, newer_tag_name2, newer_tag_time, older_tag_name, project_url) #Generate date string: time_string = newer_tag_time.strftime [:format] # Generate tag name and link log += "## [#{newer_tag_name}](#{project_url}/tree/#{newer_tag_name2}) (#{time_string})\n\n" if [:compare_link] && older_tag_name # Generate compare link log += "[Full Changelog](#{project_url}/compare/#{older_tag_name}...#{newer_tag_name2})\n\n" end log end |
#generate_log_between_tags(older_tag, newer_tag) ⇒ Object
355 356 357 358 359 360 361 362 363 364 365 366 367 368 369 370 371 372 373 374 375 |
# File 'lib/github_changelog_generator.rb', line 355 def (older_tag, newer_tag) # older_tag nil - means it's first tag, newer_tag nil - means it unreleased section filtered_pull_requests = delete_by_time(@pull_requests, :merged_at, older_tag, newer_tag) filtered_issues = delete_by_time(@issues, :actual_date, older_tag, newer_tag) newer_tag_name = newer_tag.nil? ? nil : newer_tag['name'] older_tag_name = older_tag.nil? ? nil : older_tag['name'] if [:filter_issues_by_milestone] #delete excess irrelevant issues (according milestones) filtered_issues = filter_by_milestone(filtered_issues, newer_tag_name, @issues) filtered_pull_requests = filter_by_milestone(filtered_pull_requests, newer_tag_name, @pull_requests) end if filtered_issues.empty? && filtered_pull_requests.empty? && newer_tag.nil? # do not generate empty unreleased section return nil end self.create_log(filtered_pull_requests, filtered_issues, newer_tag, older_tag_name) end |
#generate_log_for_all_tags ⇒ Object
248 249 250 251 252 253 254 255 256 257 258 259 260 261 262 263 264 265 266 267 268 269 270 271 272 273 274 275 276 277 278 279 |
# File 'lib/github_changelog_generator.rb', line 248 def if [:verbose] puts "Sorting tags.." end .sort_by! { |x| self.get_time_of_tag(x) }.reverse! if [:verbose] puts "Generating log.." end log = '' if [:unreleased] unreleased_log = self.(self.[0], nil) if unreleased_log log += unreleased_log end end (1 ... self..size).each { |index| log += self.(self.[index], self.[index-1]) } log += (nil, self..last) log end |
#generate_log_from_array(issues, prefix) ⇒ Object
508 509 510 511 512 513 514 515 516 517 518 519 520 521 |
# File 'lib/github_changelog_generator.rb', line 508 def generate_log_from_array(issues, prefix) log = '' if [:simple_list].nil? && issues.any? log += "#{prefix}\n\n" end if issues.any? issues.each { |issue| merge_string = @generator.get_string_for_issue(issue) log += "- #{merge_string}\n\n" } end log end |
#get_all_tags ⇒ Object
315 316 317 318 319 320 321 322 323 324 325 326 327 328 329 330 331 332 333 334 335 336 337 |
# File 'lib/github_changelog_generator.rb', line 315 def if [:verbose] print "Fetching tags...\r" end response = @github.repos. [:user], [:project] = [] page_i = 0 count_pages = response.count_pages response.each_page do |page| page_i += PER_PAGE_NUMBER print "Fetching tags... #{page_i}/#{count_pages * PER_PAGE_NUMBER}\r" .concat(page) end print " \r" if [:verbose] puts "Found #{tags.count} tags" end end |
#get_filtered_issues ⇒ Object
554 555 556 557 558 559 560 561 562 563 564 565 566 567 568 569 570 571 572 573 574 575 576 577 578 579 580 581 582 583 584 585 586 587 588 589 |
# File 'lib/github_changelog_generator.rb', line 554 def get_filtered_issues issues = @issues filtered_issues = issues unless [:include_labels].nil? filtered_issues = issues.select { |issue| #add all labels from @options[:incluse_labels] array (issue.labels.map { |label| label.name } & [:include_labels]).any? } end unless [:exclude_labels].nil? filtered_issues = filtered_issues.select { |issue| #delete all labels from @options[:exclude_labels] array !(issue.labels.map { |label| label.name } & [:exclude_labels]).any? } end if [:add_issues_wo_labels] issues_wo_labels = issues.select { # add issues without any labels |issue| !issue.labels.map { |label| label.name }.any? } filtered_issues |= issues_wo_labels end if [:verbose] puts "Filtered issues: #{filtered_issues.count}" end filtered_issues end |
#get_filtered_pull_requests ⇒ Object
135 136 137 138 139 140 141 142 143 144 145 146 147 148 149 150 151 152 153 154 155 156 157 158 159 160 161 162 163 164 165 166 167 168 169 170 171 172 173 174 175 176 177 178 179 180 181 182 183 184 185 186 187 188 189 190 191 192 193 194 195 196 197 198 199 200 201 202 203 204 205 206 |
# File 'lib/github_changelog_generator.rb', line 135 def get_filtered_pull_requests pull_requests = @pull_requests filtered_pull_requests = pull_requests unless [:include_labels].nil? filtered_pull_requests = pull_requests.select { |issue| #add all labels from @options[:incluse_labels] array (issue.labels.map { |label| label.name } & [:include_labels]).any? } end unless [:exclude_labels].nil? filtered_pull_requests = filtered_pull_requests.select { |issue| #delete all labels from @options[:exclude_labels] array !(issue.labels.map { |label| label.name } & [:exclude_labels]).any? } end if [:add_issues_wo_labels] issues_wo_labels = pull_requests.select { # add issues without any labels |issue| !issue.labels.map { |label| label.name }.any? } filtered_pull_requests |= issues_wo_labels end if [:verbose] puts "Filtered pull requests: #{filtered_pull_requests.count}" end filtered_pull_requests # # # # # # unless @options[:pull_request_labels].nil? # # if @options[:verbose] # puts 'Filter all pull requests by labels.' # end # # filtered_pull_requests = filtered_pull_requests.select { |pull_request| # #fetch this issue to get labels array # issue = @github.issues.get @options[:user], @options[:project], pull_request.number # # #compare is there any labels from @options[:labels] array # issue_without_labels = !issue.labels.map { |label| label.name }.any? # # if @options[:verbose] # puts "Filter request \##{issue.number}." # end # # if @options[:pull_request_labels].any? # select_by_label = (issue.labels.map { |label| label.name } & @options[:pull_request_labels]).any? # else # select_by_label = false # end # # select_by_label | issue_without_labels # } # # if @options[:verbose] # puts "Filtered pull requests with specified labels and w/o labels: #{filtered_pull_requests.count}" # end # return filtered_pull_requests # end # # filtered_pull_requests end |
#get_time_of_tag(tag_name, tag_times_hash = @tag_times_hash) ⇒ Object
539 540 541 542 543 544 545 546 547 548 549 550 551 552 |
# File 'lib/github_changelog_generator.rb', line 539 def get_time_of_tag(tag_name, tag_times_hash = @tag_times_hash) if tag_name.nil? return nil end if tag_times_hash[tag_name['name']] return @tag_times_hash[tag_name['name']] end github_git_data_commits_get = @github.git_data.commits.get [:user], [:project], tag_name['commit']['sha'] time_string = github_git_data_commits_get['committer']['date'] @tag_times_hash[tag_name['name']] = Time.parse(time_string) end |
#github_token ⇒ Object
339 340 341 342 343 344 345 346 347 348 349 350 351 352 353 |
# File 'lib/github_changelog_generator.rb', line 339 def github_token if [:token] return @github_token ||= [:token] end env_var = ENV.fetch 'CHANGELOG_GITHUB_TOKEN', nil unless env_var puts "Warning: No token provided (-t option) and variable $CHANGELOG_GITHUB_TOKEN was not found.".yellow puts "This script can make only 50 requests to GitHub API per hour without token!".yellow end @github_token ||= env_var end |
#is_megred(number) ⇒ Object
311 312 313 |
# File 'lib/github_changelog_generator.rb', line 311 def is_megred(number) @github.pull_requests.merged? [:user], [:project], number end |
#print_json(json) ⇒ Object
98 99 100 |
# File 'lib/github_changelog_generator.rb', line 98 def print_json(json) puts JSON.pretty_generate(json) end |